import mappingTypes from "./mapping-types.js";
const objectTypes = {
  Concept: {
    type: ["http://www.w3.org/2004/02/skos/core#Concept"]
  },
  ConceptScheme: {
    type: ["http://www.w3.org/2004/02/skos/core#ConceptScheme"]
  },
  ConceptMapping: {
    type: mappingTypes.map((type) => type.uri)
  },
  ConceptOccurrence: {
    type: [
      "http://purl.org/cld/cdtype/CatalogueOrIndex"
    ]
  },
  Registry: {
    type: [
      "http://www.w3.org/ns/dcat#Catalog"
    ]
  },
  Distribution: {
    type: [
      "http://www.w3.org/ns/dcat#Distribution"
    ]
  },
  Concordance: {
    type: [
      "http://rdfs.org/ns/void#Linkset",
      "http://rdf-vocabulary.ddialliance.org/xkos#Correspondence"
    ]
  },
  Resource: {},
  Item: {},
  ConceptBundle: {},
  Annotation: {
    type: [
      "Annotation",
      "http://www.w3.org/ns/oa#Annotation"
    ]
  }
};
const objectTypeUris = Object.keys(objectTypes).reduce((map, name) => {
  for (let uri of objectTypes[name].type || []) {
    map[uri] = name;
  }
  return map;
}, {});
function guessObjectType(obj, shortname = false) {
  let type;
  if (typeof obj === "string" && obj) {
    if (obj in objectTypeUris) {
      type = objectTypeUris[obj];
    } else {
      obj = obj.toLowerCase().replace(/s$/, "");
      type = Object.keys(objectTypes).find((name) => {
        const lowercase = name.toLowerCase();
        if (lowercase === obj || lowercase === "concept" + obj) {
          return true;
        }
      });
    }
  } else if (typeof obj === "object") {
    if (obj.type) {
      let types = Array.isArray(obj.type) ? obj.type : [obj.type];
      for (let uri of types) {
        if (uri in objectTypeUris) {
          type = objectTypeUris[uri];
          break;
        }
      }
    }
  }
  return shortname && type ? type.toLowerCase().replace(/^concept(.+)/, "$1") : type;
}
export {
  guessObjectType,
  objectTypes
};
